@@ -298,6 +298,9 @@ static int mroute4_del_vif(struct iface *iface)
 	if (iface->mrdisc)
 		rc = mrdisc_deregister(iface->vif);
 
+	if (iface->vif == ALL_VIFS)
+		return 0;
+
 	if (kern_vif_del(iface)) {
 		switch (errno) {
 		case ENOENT:
@@ -446,7 +449,7 @@ void mroute_expire(int max_idle)
 	clock_gettime(CLOCK_MONOTONIC, &now);
 
 	TAILQ_FOREACH_SAFE(entry, &kern_list, link, tmp) {
-		char origin[INET_ADDRSTRLEN], group[INET_ADDRSTRLEN];
+		char origin[INET_ADDRSTR_LEN], group[INET_ADDRSTR_LEN];
 		struct iface *iface;
 
 		/* XXX: only consider (*,G) routes, not pure (S,G), and no overlap handling for now */
@@ -910,6 +913,9 @@ static int mroute6_del_mif(struct iface *iface)
 {
 	int rc = 0;
 
+	if (iface->mif == ALL_VIFS)
+		return 0;
+
 	if (kern_mif_del(iface) && errno != ENOENT) {
 		switch (errno) {
 		case ENOENT:
@@ -1095,11 +1101,11 @@ void mroute_reload_end(int do_vifs)
 
 static int show_mroute(int sd, struct mroute *r, int inw, int detail)
 {
-	char src[INET_ADDRSTRLEN] = "*";
+	char src[INET_ADDRSTR_LEN] = "*";
 	char src_len[5] = "";
-	char grp[INET_ADDRSTRLEN];
+	char grp[INET_ADDRSTR_LEN];
 	char grp_len[5] = "";
-	char sg[(INET_ADDRSTRLEN + 3) * 2 + 5];
+	char sg[(INET_ADDRSTR_LEN + 3) * 2 + 5];
 	char buf[MAX_MC_VIFS * 17 + 80];
 	struct iface *iface;
 	int max_len;
